@@ -224,7 +224,7 @@ def log_model_outputs(
     ), "You must provide either logits or probs"
     assert (embs is None and exclude_embs) or (
         embs is not None and not exclude_embs
-    ), "embs can be None if and only if exclude_embs is True"
+    ), "embs can be omitted if and only if exclude_embs is True"
     if embs is None and exclude_embs:
         embs = np.random.rand(len(ids), DEFAULT_RANDOM_EMB_DIM)
     model_logger = get_model_logger()(
@@ -257,7 +257,7 @@ def set_labels_for_run(labels: Union[List[List[str]], List[str]]) -> None:
 
 
 @check_noop
-def set_tasks_for_run(tasks: List[str], binary: bool = False) -> None:
+def set_tasks_for_run(tasks: List[str], binary: bool = True) -> None:
     """Sets the task names for the run (multi-label case only).
 
     This order MUST match the order of the labels list provided in log_input_data
@@ -269,7 +269,7 @@ def set_tasks_for_run(tasks: List[str], binary: bool = False) -> None:
     :param tasks: The list of tasks for your run
     :param binary: Whether this is a binary multi label run. If true, tasks will also
     be set as your labels, and you should NOT call `dq.set_labels_for_run` it will be
-    handled for you. Default False
+    handled for you. Default True
     """
     if config.task_type != TaskType.text_multi_label:
         raise GalileoException("You can only set task names for multi-label use cases.")
