@@ -6,6 +6,7 @@ from typing import Any, DefaultDict, Dict, List, Optional, Tuple, Union
 import numpy as np
 from numpy import ndarray
 
+from dataquality.exceptions import LogBatchError
 from dataquality.loggers.logger_config.text_ner import text_ner_logger_config
 from dataquality.loggers.model_logger.base_model_logger import BaseGalileoModelLogger
 from dataquality.schemas import __data_schema_version__
@@ -140,7 +141,7 @@ class TextNERModelLogger(BaseGalileoModelLogger):
         * embs, probs, and ids must exist and be the same length
         :return:
         """
-        get_dq_logger().info(
+        get_dq_logger().debug(
             "Validating the output log.", split=self.split, epoch=self.epoch
         )
         if len(self.logits):
@@ -174,10 +175,11 @@ class TextNERModelLogger(BaseGalileoModelLogger):
                 logged_sample_ids.append(sample_id)
 
         self.ids = logged_sample_ids
-        assert self.ids, (
-            "No samples in this batch had any gold or prediction spans. Logging will "
-            "be skipped"
-        )
+        if not self.ids:
+            raise LogBatchError(
+                "No samples in this batch had any gold or prediction spans. "
+                "Logging will be skipped"
+            )
 
     def _process_sample(
         self, sample_id: int, sample_emb: np.ndarray, sample_prob: np.ndarray
@@ -186,8 +188,12 @@ class TextNERModelLogger(BaseGalileoModelLogger):
 
         A sample should be logged only if there was at least 1 prediction span or 1
         gold span
+
+        For inference mode, only prediction spans should be logged.
         """
-        get_dq_logger().info("Processing a sample.", split=self.split, epoch=self.epoch)
+        get_dq_logger().debug(
+            "Processing a sample.", split=self.split, epoch=self.epoch
+        )
         # To extract metadata about the sample we are looking at
         sample_key = self.logger_config.get_sample_key(Split(self.split), sample_id)
 
@@ -197,27 +203,35 @@ class TextNERModelLogger(BaseGalileoModelLogger):
         # Get prediction spans
         sample_pred_spans = self._extract_pred_spans(sample_prob, sample_token_len)
         # Get gold (ground truth) spans
-        gold_span_tup = self.logger_config.gold_spans.get(sample_key, [])
-        sample_gold_spans: List[Dict] = [
+        gold_span_tup = []
+        if self.split != Split.inference:
+            gold_span_tup = self.logger_config.gold_spans.get(sample_key, [])
+
+        sample_gold_spans = [
             dict(start=start, end=end, label=label)
             for start, end, label in gold_span_tup
         ]
+
         # If there were no golds and no preds for a sample, don't log this sample
         if not sample_pred_spans and not sample_gold_spans:
             return False
 
-        gold_dep, pred_dep = self._calculate_dep_scores(
-            sample_prob, sample_gold_spans, sample_pred_spans, sample_token_len
-        )
-        gold_emb = self._extract_span_embeddings(sample_gold_spans, sample_emb)
         pred_emb = self._extract_span_embeddings(sample_pred_spans, sample_emb)
-
         self.pred_spans.append(sample_pred_spans)
-        self.gold_spans.append(sample_gold_spans)
-        self.pred_dep.append(pred_dep)
-        self.gold_dep.append(gold_dep)
-        self.gold_emb.append(gold_emb)
         self.pred_emb.append(pred_emb)
+
+        if self.split != Split.inference:
+            # If we are not in inference mode, we also have gold spans and DEP
+            gold_emb = self._extract_span_embeddings(sample_gold_spans, sample_emb)
+            self.gold_spans.append(sample_gold_spans)
+            self.gold_emb.append(gold_emb)
+
+            gold_dep, pred_dep = self._calculate_dep_scores(
+                sample_prob, sample_gold_spans, sample_pred_spans, sample_token_len
+            )
+            self.gold_dep.append(gold_dep)
+            self.pred_dep.append(pred_dep)
+
         return True
 
     def _extract_span_embeddings(
@@ -228,7 +242,7 @@ class TextNERModelLogger(BaseGalileoModelLogger):
         We take the average of the token embeddings per span and use that as the span
         level embedding
         """
-        get_dq_logger().info(
+        get_dq_logger().debug(
             "Extracting span embeddings.", split=self.split, epoch=self.epoch
         )
         embeddings = []
@@ -248,7 +262,7 @@ class TextNERModelLogger(BaseGalileoModelLogger):
         """
         # use length of the tokens stored to strip the pads
         # Drop the spans post first PAD
-        get_dq_logger().info(
+        get_dq_logger().debug(
             "Extracting pred spans.", split=self.split, epoch=self.epoch
         )
         argmax_indices: List[int] = np.array(pred_prob).argmax(axis=1)
@@ -257,14 +271,12 @@ class TextNERModelLogger(BaseGalileoModelLogger):
         ][0:sample_len]
 
         if self.logger_config.tagging_schema == TaggingSchema.BIO:
-            pred_spans = self._extract_pred_spans_bio(pred_sequence)
-        elif self.logger_config.tagging_schema == TaggingSchema.BILOU:
-            pred_spans = self._extract_pred_spans_bilou(pred_sequence)
-        else:  # BIOES
-            pred_spans = self._extract_pred_spans_bioes(pred_sequence)
+            pred_spans = self._extract_spans_bio(pred_sequence)
+        else:  # BIOES or BILOU
+            pred_spans = self._extract_spans_token_level(pred_sequence)
         return pred_spans
 
-    def _extract_pred_spans_bio(self, pred_sequence: List[str]) -> List[Dict]:
+    def _extract_spans_bio(self, pred_sequence: List[str]) -> List[Dict]:
         """
         Converts the prediction sequences into prediction span tokens
 
@@ -328,27 +340,26 @@ class TextNERModelLogger(BaseGalileoModelLogger):
         assert total_b_count == len(pred_spans)
         return pred_spans
 
-    def _extract_pred_spans_bilou(self, pred_sequence: List[str]) -> List[Dict]:
-        """BILOU is a special case for BIO.
+    def _extract_spans_token_level(self, sequence: List[str]) -> List[Dict]:
+        """Extract spans at token or word level for gold or pred spans.
 
-        The presense of I in a sequence does not mean a
-        presence of a span until an L is successfully predicted.
+        This should be called with a BILOU or BIOES sequence
         """
-        pred_spans = []
+        spans = []
         total_b_count = 0
         idx = 0
         found_end = False  # Tracks if there was an end tag predicted for BIOES
 
         # Use a while loop so we can skip rows already scanned in the inner loop
-        while idx < len(pred_sequence):
-            token = pred_sequence[idx]
+        while idx < len(sequence):
+            token = sequence[idx]
             next_idx = idx + 1
 
             if self._is_single_token(token):
                 total_b_count += 1
                 # hit a single token prediction, update and continue
                 token_val, token_label = self._split_token(token)
-                pred_spans.append({"start": idx, "end": next_idx, "label": token_label})
+                spans.append({"start": idx, "end": next_idx, "label": token_label})
                 idx += 1
                 continue
 
@@ -361,63 +372,11 @@ class TextNERModelLogger(BaseGalileoModelLogger):
             # different label
             token_val, token_label = self._split_token(token)
 
-            for next_tok in pred_sequence[idx + 1 :]:
-                if self._is_in_token(next_tok, token_label):
-                    next_idx += 1
-                # next_tok == "L" and the label matches the current B label
-                elif self._is_end_token(next_tok, token_label):
-                    next_idx += 1
-                    found_end = True
-                    total_b_count += 1
-                    break
-                else:
-                    break
-            if found_end:
-                pred_spans.append({"start": idx, "end": next_idx, "label": token_label})
-            idx = next_idx
-            found_end = False
-
-        assert total_b_count == len(pred_spans)
-        return pred_spans
-
-    def _extract_pred_spans_bioes(self, pred_sequence: List[str]) -> List[Dict]:
-        """BIOES is a special case for BIO.
-
-        The presense of I in a sequence does not mean a presence of a span until
-        an E is successfully predicted.
-        """
-        pred_spans = []
-        total_b_count = 0
-        idx = 0
-        found_end = False  # Tracks if there was an end tag predicted for BIOES
-
-        # Use a while loop so we can skip rows already scanned in the inner loop
-        while idx < len(pred_sequence):
-            token = pred_sequence[idx]
-            next_idx = idx + 1
-
-            if self._is_single_token(token):
-                total_b_count += 1
-                # hit a single token prediction , update and continue
-                token_val, token_label = self._split_token(token)
-                pred_spans.append({"start": idx, "end": next_idx, "label": token_label})
-                idx += 1
-                continue
-
-            if not self._is_before_token(token):
-                idx += 1
-                continue
-
-            # We've hit a prediction. Continue until it's invalid
-            # Invalid means we hit a new B or O tag, or the next I tag has a
-            # different label
-            token_val, token_label = self._split_token(token)
-
-            for next_tok in pred_sequence[idx + 1 :]:
+            for next_tok in sequence[idx + 1 :]:
                 # next_tok == "I" and the label matches the current B label
                 if self._is_in_token(next_tok, token_label):
                     next_idx += 1
-                # next_tok == "E" and the label matches the current B label
+                # next_tok == "L"/"E" and the label matches the current B label
                 elif self._is_end_token(next_tok, token_label):
                     next_idx += 1
                     found_end = True
@@ -426,12 +385,12 @@ class TextNERModelLogger(BaseGalileoModelLogger):
                 else:
                     break
             if found_end:
-                pred_spans.append({"start": idx, "end": next_idx, "label": token_label})
+                spans.append({"start": idx, "end": next_idx, "label": token_label})
             idx = next_idx
             found_end = False
 
-        assert total_b_count == len(pred_spans)
-        return pred_spans
+        assert total_b_count == len(spans)
+        return spans
 
     def _is_single_token(self, tok: str) -> bool:
         return tok.startswith("U") or tok.startswith("S")
@@ -469,7 +428,7 @@ class TextNERModelLogger(BaseGalileoModelLogger):
         gold_sequence = [O, O, O, O, O, B-nothing, I-nothing, I-nothing, O, O] for BIO
         gold_sequence = [O, O, O, O, O, B-nothing, I-nothing, L-nothing, O, O] for BILOU
         """
-        get_dq_logger().info(
+        get_dq_logger().debug(
             "Constructing a gold sequence", split=self.split, epoch=self.epoch
         )
         gold_sequence = ["O"] * len_sequence
@@ -505,7 +464,7 @@ class TextNERModelLogger(BaseGalileoModelLogger):
         dep_scores: DEP scores for every token in a sample, so len(dep_scores) is
             the number of tokens in a sentence
         """
-        get_dq_logger().info(
+        get_dq_logger().debug(
             "Calculating DEP across spans.", split=self.split, epoch=self.epoch
         )
         dep_score_per_span = []
@@ -535,7 +494,7 @@ class TextNERModelLogger(BaseGalileoModelLogger):
             predicted spans which are padded by the model
         :return: The DEP score per-token for both the gold spans and pred spans
         """
-        get_dq_logger().info("Calculating DEP.", split=self.split, epoch=self.epoch)
+        get_dq_logger().debug("Calculating DEP.", split=self.split, epoch=self.epoch)
         pred_prob = np.array(pred_prob)
         label2idx = {l: i for i, l in enumerate(self.logger_config.labels)}
         argmax_indices: List[int] = pred_prob.argmax(axis=1).tolist()
@@ -559,6 +518,7 @@ class TextNERModelLogger(BaseGalileoModelLogger):
             dep = (1 - aum) / 2  # normalize aum to dep
             assert 1.0 >= dep >= 0.0, f"DEP score is out of bounds with value {dep}"
             dep_scores_tokens.append(dep)
+
         assert sample_token_len == len(
             dep_scores_tokens
         ), "misalignment between total tokens and DEP scores"
@@ -577,10 +537,16 @@ class TextNERModelLogger(BaseGalileoModelLogger):
         through the data twice, once for prediction span information
         (one of pred_span, pred_emb, pred_dep per span) and once for gold span
         information (one of gold_span, gold_emb, gold_dep per span)
+
+        NOTE: All spans are at the token level in this fn
         """
+        get_dq_logger().debug("Getting data dict.", split=self.split, epoch=self.epoch)
+
+        if self.split == Split.inference:
+            return self._get_data_dict_inference()
+
         data: defaultdict = defaultdict(list)
 
-        get_dq_logger().info("Getting data dict.", split=self.split, epoch=self.epoch)
         # Loop through samples
         num_samples = len(self.ids)
         for idx in range(num_samples):
@@ -634,7 +600,39 @@ class TextNERModelLogger(BaseGalileoModelLogger):
             # Loop through the remaining pred spans
             for pred_span, pred_emb, pred_dep in zip(pred_spans, pred_embs, pred_deps):
                 data = self._construct_pred_span_row(
-                    data, sample_id, pred_span, pred_emb, pred_dep, gold_spans
+                    data,
+                    sample_id,
+                    pred_span,
+                    pred_emb,
+                    pred_dep=pred_dep,
+                    gold_spans=gold_spans,
+                )
+        return data
+
+    def _get_data_dict_inference(self) -> Dict[str, Any]:
+        """Format row data for inference NER
+
+        In inference NER, we don't have gold spans so we only need to
+        assemble the data for pred spans.
+
+        NOTE: All spans are at the token level in this fn
+        """
+        data: defaultdict = defaultdict(list)
+
+        # Loop through samples
+        num_samples = len(self.ids)
+        for idx in range(num_samples):
+            sample_id = self.ids[idx]
+            pred_spans = self.pred_spans[idx]
+            pred_embs = self.pred_emb[idx]
+
+            # Loop through the remaining pred spans
+            for pred_span, pred_emb in zip(pred_spans, pred_embs):
+                data = self._construct_pred_span_row(
+                    data,
+                    sample_id,
+                    pred_span,
+                    pred_emb,
                 )
         return data
 
@@ -648,8 +646,13 @@ class TextNERModelLogger(BaseGalileoModelLogger):
     ) -> DefaultDict:
         span_ind = (gold_span["start"], gold_span["end"])
         data = self._construct_span_row(
-            data, sample_id, span_ind[0], span_ind[1], gold_dep, gold_emb
+            d=data,
+            id=sample_id,
+            start=span_ind[0],
+            end=span_ind[1],
+            emb=gold_emb,
         )
+        data["data_error_potential"].append(gold_dep)
         data["is_gold"].append(True)
         data["gold"].append(gold_span["label"])
         return data
@@ -660,22 +663,37 @@ class TextNERModelLogger(BaseGalileoModelLogger):
         sample_id: int,
         pred_span: Dict,
         pred_emb: np.ndarray,
-        pred_dep: float,
-        gold_spans: List[Dict],
+        pred_dep: Optional[float] = None,
+        gold_spans: Optional[List[Dict]] = None,
     ) -> DefaultDict:
         start, end = pred_span["start"], pred_span["end"]
-        data = self._construct_span_row(data, sample_id, start, end, pred_dep, pred_emb)
-        data["is_gold"].append(False)
+        data = self._construct_span_row(
+            d=data,
+            id=sample_id,
+            start=start,
+            end=end,
+            emb=pred_emb,
+        )
         data["is_pred"].append(True)
         data["pred"].append(pred_span["label"])
-        data["gold"].append("")
-        # Pred only spans are known as "ghost" spans (hallucinated) or no error
-        err = (
-            NERErrorType.ghost_span.value
-            if self._is_ghost_span(pred_span, gold_spans)
-            else NERErrorType.none.value
-        )
-        data["galileo_error_type"].append(err)
+
+        if self.split == Split.inference and self.inference_name:
+            data["inference_name"].append(self.inference_name)
+        else:
+            assert (
+                gold_spans is not None
+            ), f"gold_spans must be provided for split {self.split}"
+            data["data_error_potential"].append(pred_dep)
+            data["is_gold"].append(False)
+            data["gold"].append("")
+            # Pred only spans are known as "ghost" spans (hallucinated) or no error
+            err = (
+                NERErrorType.ghost_span.value
+                if self._is_ghost_span(pred_span, gold_spans)
+                else NERErrorType.none.value
+            )
+            data["galileo_error_type"].append(err)
+
         return data
 
     def _is_ghost_span(self, pred_span: Dict, gold_spans: List[Dict]) -> bool:
@@ -698,7 +716,12 @@ class TextNERModelLogger(BaseGalileoModelLogger):
         return True
 
     def _construct_span_row(
-        self, d: DefaultDict, id: int, start: int, end: int, dep: float, emb: ndarray
+        self,
+        d: DefaultDict,
+        id: int,
+        start: int,
+        end: int,
+        emb: ndarray,
     ) -> DefaultDict:
         d["sample_id"].append(id)
         d["epoch"].append(self.epoch)
@@ -706,7 +729,6 @@ class TextNERModelLogger(BaseGalileoModelLogger):
         d["data_schema_version"].append(__data_schema_version__)
         d["span_start"].append(start)
         d["span_end"].append(end)
-        d["data_error_potential"].append(dep)
         d["emb"].append(emb)
         return d
 
@@ -722,7 +744,7 @@ class TextNERModelLogger(BaseGalileoModelLogger):
         In this function, we only look at spans that don't have pred/gold alignment,
         so the error can only be span_shift or missed_label.
         """
-        get_dq_logger().info(
+        get_dq_logger().debug(
             "Getting span error type.", split=self.split, epoch=self.epoch
         )
         gold_start, gold_end = gold_span
@@ -753,7 +775,7 @@ class TextNERModelLogger(BaseGalileoModelLogger):
         """Converts logits to probs via softmax per sample"""
         # axis ensures that in a matrix of probs with dims num_samples x num_classes
         # we take the softmax for each sample
-        get_dq_logger().info(
+        get_dq_logger().debug(
             "Converting logits to probs.", split=self.split, epoch=self.epoch
         )
         token_probs = []
